@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileCopy.cc,v 1.162 2011/04/22 15:43:11 lav Exp $ */
-
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
     2) tries to read some data at seek_pos, sets pos to position of Get (get).
@@ -253,7 +251,7 @@ int FileCopy::Do()
 	    return MOVED;
 	 }
       }
-      if(put->Size()>max_buf)
+      if(put->IsFull())
 	 get->Suspend(); // stall the get.
       get->Get(&b,&s);
       if(b==0) // eof
@@ -336,6 +334,11 @@ int FileCopy::Do()
 	 put->Put(b,s);
 	 line_buffer->Skip(s);
       }
+      if(!CheckFileSizeAtEOF())
+      {
+	 SetError(_("file size decreased during transfer"));
+	 return MOVED;
+      }
    pre_CONFIRM_WAIT:
       put->SetSuggestedFileName(get->GetSuggestedFileName());
       put->SetDate(get->GetDate());
@@ -410,13 +413,15 @@ int FileCopy::Do()
 
 FileCopy::FileCopy(FileCopyPeer *s,FileCopyPeer *d,bool c)
    : get(s), put(d), cont(c),
-   rate(new Speedometer("xfer:rate-period")),
-   rate_for_eta(new Speedometer("xfer:eta-period"))
+   rate("xfer:rate-period"),
+   rate_for_eta("xfer:eta-period")
 {
    set_state(INITIAL);
-   max_buf=buffer_size.Query(0);
+   int max_buf=buffer_size.Query(0);
    if(max_buf<1)
       max_buf=1;
+   s->SetMaxBuffered(max_buf);
+   d->SetMaxBuffered(max_buf);
    put_buf=0;
    put_eof_pos=0;
    bytes_count=0;
@@ -440,6 +445,7 @@ FileCopy *FileCopy::New(FileCopyPeer *s,FileCopyPeer *d,bool c)
 }
 void FileCopy::SuspendInternal()
 {
+   super::SuspendInternal();
    if(get) get->SuspendSlave();
    if(put) put->SuspendSlave();
 }
@@ -447,6 +453,7 @@ void FileCopy::ResumeInternal()
 {
    if(get) get->ResumeSlave();
    if(put) put->ResumeSlave();
+   super::ResumeInternal();
 }
 void FileCopy::Fg()
 {
@@ -473,7 +480,7 @@ void FileCopy::LineBuffered(int s)
    line_buffer_max=s;
 }
 
-off_t FileCopy::GetPos()
+off_t FileCopy::GetPos() const
 {
    if(put)
       return put->GetRealPos() - put->Buffered();
@@ -482,14 +489,14 @@ off_t FileCopy::GetPos()
    return 0;
 }
 
-off_t FileCopy::GetSize()
+off_t FileCopy::GetSize() const
 {
    if(get)
       return get->GetSize();
    return NO_SIZE;
 }
 
-int FileCopy::GetPercentDone()
+int FileCopy::GetPercentDone() const
 {
    if(!get || !put)
       return 100;
@@ -510,7 +517,7 @@ int FileCopy::GetPercentDone()
       return -1;
    return percent(ppos,psize);
 }
-const char *FileCopy::GetPercentDoneStr()
+const char *FileCopy::GetPercentDoneStr() const
 {
    int pct=GetPercentDone();
    if(pct==-1)
@@ -521,26 +528,26 @@ const char *FileCopy::GetPercentDoneStr()
 }
 void FileCopy::RateAdd(int a)
 {
-   rate->Add(a);
-   rate_for_eta->Add(a);
+   rate.Add(a);
+   rate_for_eta.Add(a);
 }
 void FileCopy::RateReset()
 {
    start_time=now;
-   rate->Reset();
-   rate_for_eta->Reset();
+   rate.Reset();
+   rate_for_eta.Reset();
 }
 float FileCopy::GetRate()
 {
-   if(!rate->Valid() || !put)
+   if(!rate.Valid() || !put)
       return 0;
-   return rate->Get();
+   return rate.Get();
 }
 const char *FileCopy::GetRateStr()
 {
-   if(!rate->Valid() || !put)
+   if(!rate.Valid() || !put)
       return "";
-   return rate->GetStrS();
+   return rate.GetStrS();
 }
 off_t FileCopy::GetBytesRemaining()
 {
@@ -549,7 +556,7 @@ off_t FileCopy::GetBytesRemaining()
    if(get->range_limit==FILE_END)
    {
       off_t size=get->GetSize();
-      if(size<=0 || size<get->GetRealPos() || !rate_for_eta->Valid())
+      if(size<=0 || size<get->GetRealPos() || !rate_for_eta.Valid())
 	 return -1;
       return(size-GetPos());
    }
@@ -560,13 +567,13 @@ const char *FileCopy::GetETAStr()
    off_t b=GetBytesRemaining();
    if(b<0 || !put)
       return "";
-   return rate_for_eta->GetETAStrSFromSize(b);
+   return rate_for_eta.GetETAStrSFromSize(b);
 }
 long FileCopy::GetETA(off_t b)
 {
-   if(b<0 || !rate_for_eta->Valid())
+   if(b<0 || !rate_for_eta.Valid())
       return -1;
-   return (long)(double(b) / rate_for_eta->Get() + 0.5);
+   return (long)(double(b) / rate_for_eta.Get() + 0.5);
 }
 const char *FileCopy::GetStatus()
 {
@@ -630,10 +637,11 @@ void FileCopy::LogTransfer()
    {
       const char *fname = ResMgr::Query("xfer:log-file", 0);
       if(!fname || !*fname)
-	 fname = dir_file(get_lftp_home(),"transfer_log");
+	 fname = dir_file(get_lftp_data_dir(),"transfer_log");
       int fd = open(fname,O_WRONLY|O_APPEND|O_CREAT,0600);
       if(fd==-1)
 	 return;
+      fcntl(fd,F_SETFD,FD_CLOEXEC);
       transfer_log=new Log;
       transfer_log->SetOutput(fd,true);
       transfer_log->ShowNothing();
@@ -646,7 +654,7 @@ void FileCopy::LogTransfer()
    transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",
       url::remove_password(src),url::remove_password(dst),
       (long long)GetRangeStart(),range_limit,
-      Speedometer::GetStr(GetBytesCount()/GetTimeSpent()).get());
+      Speedometer::GetStrProper(GetBytesCount()/GetTimeSpent()).get());
 }
 
 void FileCopy::SetRange(off_t s,off_t lim)
@@ -655,6 +663,26 @@ void FileCopy::SetRange(off_t s,off_t lim)
    put->SetRange(s,lim);
 }
 
+bool FileCopy::CheckFileSizeAtEOF() const
+{
+   long long range_limit=GetRangeLimit();
+   if(range_limit==FILE_END)
+   {
+      const long long size=GetSize();
+      if(size==NO_SIZE || size==NO_SIZE_YET)
+	 return true;   // nothing to compare with.
+      range_limit=size;
+   }
+
+   const long long get_pos=get->GetRealPos();
+   const long long put_pos=put->GetRealPos();
+   const long long pos=(get_pos>put_pos ? get_pos : put_pos);
+   if(pos>=range_limit)
+      return true;
+
+   debug((0,"expected pos=%lld, actual pos=%lld\n",range_limit,pos));
+   return false;
+}
 
 // FileCopyPeer implementation
 #undef super
@@ -726,6 +754,7 @@ FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
    want_date=false;
    start_transfer=true;
    size=NO_SIZE_YET;
+   date=NO_DATE_YET;
    e_size=NO_SIZE;
    seek_pos=0;
    can_seek=false;
@@ -782,16 +811,23 @@ int FileCopyPeerFA::Do()
       return m;
    }
 
-   if(want_size && size==NO_SIZE_YET && (mode==PUT || !start_transfer))
+   // if we need some info and cannot start the transfer (yet),
+   // then use ARRAY_INFO to fetch the file information.
+   if(((want_size && size==NO_SIZE_YET) || (want_date && date==NO_DATE_YET))
+   && (mode==PUT || !start_transfer) && session->IsClosed())
+   {
+      FileInfo *fi=new FileInfo(file);
+      if(want_size)
+	 fi->Need(fi->SIZE);
+      if(want_date)
+	 fi->Need(fi->DATE);
+      info.Empty();
+      info.Add(fi);
+      session->GetInfoArray(&info);
+      m=MOVED;
+   }
+   if(session->OpenMode()==FA::ARRAY_INFO)
    {
-      if(session->IsClosed())
-      {
-	 info.file=file;
-	 info.get_size=true;
-	 info.get_time=want_date;
-	 session->GetInfoArray(&info,1);
-	 m=MOVED;
-      }
       res=session->Done();
       if(res==FA::IN_PROGRESS)
 	 return m;
@@ -799,10 +835,14 @@ int FileCopyPeerFA::Do()
       {
 	 session->Close();
 	 SetSize(NO_SIZE);
+	 SetDate(NO_DATE);
 	 return MOVED;
       }
-      SetSize(info.size);
-      SetDate(info.time);
+      FileInfo *fi=info[0];
+      if(want_size)
+	 SetSize(fi->size);
+      if(want_date)
+	 SetDate(fi->date);
       session->Close();
       return MOVED;
    }
@@ -914,9 +954,11 @@ void FileCopyPeerFA::SuspendInternal()
       return;
    if(session->IsOpen())
       session->SuspendSlave();
+   super::SuspendInternal();
 }
 void FileCopyPeerFA::ResumeInternal()
 {
+   super::ResumeInternal();
    session->ResumeSlave();
 }
 
@@ -1516,7 +1558,9 @@ void FileCopyPeerFDStream::Seek(off_t new_pos)
    {
       if(seek_pos!=FILE_END)
       {
-	 pos=seek_pos+(mode==PUT?Size():0);
+	 pos=seek_pos;
+	 if(mode==PUT)
+	    pos+=Size();
 	 return;
       }
       else
@@ -1817,6 +1861,21 @@ int FileCopyPeerDirList::Do()
    return MOVED;
 }
 
+// FileCopyPeerMemory
+int FileCopyPeerMemory::Do()
+{
+   int m=STALL;
+   if(mode==PUT) {
+      max_buf=max_size+1;
+      if(Size()>max_size) {
+	 SetError("buffer limit exceeded");
+	 broken=true;
+	 return MOVED;
+      }
+   }
+   return m;
+}
+
 // FileVerificator
 void FileVerificator::Init0()
 {
