@@ -701,10 +701,14 @@ void Http::SendRequest(const char *connection,const char *f)
       break;
 
    case(REMOVE):
-   case(REMOVE_DIR):
       SendMethod("DELETE",efile);
-      if(mode==REMOVE)
-	 Send("Depth: 0\r\n"); // deny directory removal
+      Send("Depth: 0\r\n"); // deny directory removal
+      break;
+
+   case(REMOVE_DIR):
+      if(efile.last_char()!='/')
+	 efile.append('/');
+      SendMethod("DELETE",efile);
       break;
 
    case ARRAY_INFO:
@@ -863,7 +867,8 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       if(pos==0 && opt_size && H_2XX(status_code) && !inflate)
 	 *opt_size=body_size;
 
-      if(mode==ARRAY_INFO && H_2XX(status_code) && !propfind)
+      if(mode==ARRAY_INFO && H_2XX(status_code)
+      && xstrcmp(last_method,"PROPFIND"))
       {
 	 FileInfo *fi=fileset_for_info->curr();
 	 fi->SetSize(body_size);
@@ -1475,7 +1480,7 @@ int Http::Do()
 		  if(H_2XX(status_code))
 		  {
 		     // should never happen
-		     LogError(0,"Success, but did nothing??");
+		     LogError(0,"Unexpected success, the server did not accept full request body");
 		     Disconnect();
 		     return MOVED;
 		  }
@@ -2475,6 +2480,7 @@ check_end (const char *p)
     ++p;
   if (!*p
       || (p[0] == 'G' && p[1] == 'M' && p[2] == 'T')
+      || (p[0] == 'U' && p[1] == 'T' && p[2] == 'C')
       || ((p[0] == '+' || p[1] == '-') && ISDIGIT (p[1])))
     return 1;
   else
