@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: HttpDir.cc,v 1.108 2011/03/25 14:35:42 lav Exp $ */
-
 #include <config.h>
 #include "ascii_ctype.h"
 #include <assert.h>
@@ -380,7 +378,6 @@ static bool try_apache_unixlike(file_info &info,const char *buf,
 
    // Apache Unix-like listing (from apache proxy):
    //   Perms Nlnk user [group] size Mon DD (YYYY or hh:mm)
-   int perms_code;
    char year_or_time[6];
    int consumed;
 
@@ -393,7 +390,7 @@ static bool try_apache_unixlike(file_info &info,const char *buf,
       n=sscanf(buf,"%11s %d %31s %lld %3s %2d %5s%n",info.perms,&info.nlink,
 	    info.user,&info.size,info.month_name,&info.day,year_or_time,&consumed);
    }
-   if(n>=7 && -1!=(perms_code=parse_perms(info.perms+1))
+   if(n>=7 && -1!=parse_perms(info.perms+1)
    && -1!=(info.month=parse_month(info.month_name))
    && -1!=parse_year_or_time(year_or_time,&info.year,&info.hour,&info.minute))
    {
@@ -501,7 +498,6 @@ static bool try_wwwoffle_ftp(file_info &info,const char *buf,
    info.clear();
 
    //   Perms Nlnk user [group] size Mon DD (YYYY or hh:mm)
-   int perms_code;
    char year_or_time[6];
    int consumed;
 
@@ -514,7 +510,7 @@ static bool try_wwwoffle_ftp(file_info &info,const char *buf,
       n=sscanf(buf,"%11s %d %31s %lld %3s %2d %5s%n",info.perms,&info.nlink,
 	    info.user,&info.size,info.month_name,&info.day,year_or_time,&consumed);
    }
-   if(n>=7 && -1!=(perms_code=parse_perms(info.perms+1))
+   if(n>=7 && -1!=parse_perms(info.perms+1)
    && -1!=(info.month=parse_month(info.month_name))
    && -1!=parse_year_or_time(year_or_time,&info.year,&info.hour,&info.minute))
    {
@@ -1364,6 +1360,7 @@ const char *HttpDirList::Status()
 
 void HttpDirList::SuspendInternal()
 {
+   super::SuspendInternal();
    if(ubuf)
       ubuf->SuspendSlave();
 }
@@ -1371,8 +1368,9 @@ void HttpDirList::ResumeInternal()
 {
    if(ubuf)
       ubuf->ResumeSlave();
+   super::ResumeInternal();
 }
-
+#undef super
 
 // HttpListInfo implementation
 FileSet *HttpListInfo::Parse(const char *b,int len)
@@ -1392,6 +1390,9 @@ FileSet *Http::ParseLongList(const char *b,int len,int *err) const
    if(err)
       *err=0;
 
+   if(len>5 && !strncmp(b,"<?xml",5))
+      return HttpListInfo::ParseProps(b,len,GetCwd());
+
    FileSet *set=new FileSet;
    ParsedURL prefix(GetConnectURL());
    xstring_c base_href;
