@@ -123,7 +123,7 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    static SMTaskRef<TorrentListener> listener_ipv6_udp;
    static SMTaskRef<DHT> dht_ipv6;
 #endif
-   static Ref<FDCache> fd_cache;
+   static SMTaskRef<FDCache> fd_cache;
    static Ref<TorrentBlackList> black_list;
 
    static const SMTaskRef<DHT>& GetDHT(int af)
@@ -256,8 +256,11 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    Timer optimistic_unchoke_timer;
    Timer peers_scan_timer;
    Timer am_interested_timer;
+   Timer shutting_down_timer;
 
    Timer dht_announce_timer;
+   int dht_announce_count;
+   int dht_announce_count_ipv6;
 
    static const int max_uploaders = 20;
    static const int min_uploaders = 1;
@@ -352,6 +355,27 @@ public:
       if(dht)
 	 dht->AddBootstrapNode(n);
    }
+
+   static bool HasDHT() {
+#if INET6
+      if(dht_ipv6)
+	 return true;
+#endif
+      if(dht)
+	 return true;
+      return false;
+   }
+   static bool HasDHT(int af) {
+#if INET6
+      if(af==AF_INET6 && dht_ipv6)
+	 return true;
+#endif
+      if(af==AF_INET && dht)
+	 return true;
+      return false;
+   }
+   const char *DHT_Status() const;
+   void DHT_Announced(int af);	 // called from DHT to count announces
 };
 
 class FDCache : public SMTask, public ResClient
@@ -721,7 +745,7 @@ public:
    bool Disconnected() const { return passive && NotConnected(); }
    bool Connected() const { return peer_id && send_buf && recv_buf; }
    bool Active() const { return Connected() && (am_interested || peer_interested); }
-   bool Complete() const { return peer_complete_pieces==parent->total_pieces; }
+   bool Complete() const { return peer_complete_pieces==parent->total_pieces && parent->total_pieces>0; }
    bool Seed() const { return Complete() || upload_only; }
    bool AddressEq(const TorrentPeer *o) const;
    bool IsPassive() const { return passive; }
